	;; ecp.asm
	;; 
	;; ECP-Protocol for the MSP430
	;;
	cpu msp430
	include regmsp.inc
	relaxed on
	padding off

RUNSTATUS equ	1
ECPDEBUG equ	0
	
SYSTEMTAKT	equ	7000000
DMXBAUDRATE	equ	250000
		
;; Register Defs
out0i	define	R4
out1i	define	R5
in0i	define	R6
in1i	define	R7

ecpouti	define	R8

	if ECPDEBUG
ecpdebreg define R13
	endif
	
reg0	define	R14		; temporary register which is undefined after funtion calls
returnreg define R15		; return value from function calls

;; Buffer Defs

out0buf	equ	0x200
out1buf	equ	0x400
in0buf	equ	0x600
in1buf	equ	0x800

stacklen equ	4*8		; 8 calls each using 4 bytes
stackstart equ	0x9EC

led1count equ	0x9EC
led2count equ	0x9EE
dmxconfig equ	0x9F0
out0sc	equ	0x9F2
out1sc	equ	0x9F3	
out0max	equ	0x9F4
out1max	equ	0x9F6
out0brlen equ	0x9F8
out1brlen equ	0x9FA
out0mklen equ	0x9FC
out1mklen equ	0x9FE
	
;; Bit Defs

DMXOUT0ACTIVE	equ	0x01
DMXOUT1ACTIVE	equ	0x02
DMXIN0ACTIVE	equ	0x04
DMXIN0MERGETO0	equ	0x08
DMXIN1ACTIVE	equ	0x10
DMXIN1MERGETO0	equ	0x20	
	
GIE	equ	0x08	
CPUOff	equ	0x10
OscOff	equ	0x20
SCG0	equ	0x40
SCG1	equ	0x80
	
WDTIE	equ	0x01
OFIE	equ	0x02
NMIIE	equ	0x10
ACCVIE	equ	0x20
U0RXIE	equ	0x40
U0TXIE	equ	0x80
U1RXIE	equ	0x10
U1TXIE	equ	0x20

U0RXE	equ	0x40
U0TXE	equ	0x80
U1RXE	equ	0x10
U1TXE	equ	0x20

XT2OFF	equ	0x80
XTS	equ	0x40
CHAR	equ	0x10
SSEL0	equ	0x10
SSEL1	equ	0x20

SELM0	equ	0x40
SELM1	equ	0x80

UTXIFG0	equ	0x80
OFIFG	equ	0x02

TAIFG	equ	0x01
TAIE	equ	0x02
TACLR	equ	0x04
TAMC0	equ	0x10
TAMC1	equ	0x20
TAID0	equ	0x40
TAID1	equ	0x80
TASSEL0	equ	0x100
TASSEL1	equ	0x200

TACCIFG	equ	0x01
TACOV	equ	0x02
TAOUT	equ	0x04
TACCI	equ	0x08
TACCIE	equ	0x10
TAOUTMOD0 equ	0x20
TAOUTMOD1 equ	0x40
TAOUTMOD2 equ	0x80
TACAP	equ	0x100
TASCCI	equ	0x400
TASCS	equ	0x800
TACCIS0	equ	0x1000
TACCIS1	equ	0x2000
TACM0	equ	0x4000
TACM1	equ	0x8000
	
;;; ECP defs
ECPDATAIN	equ	p5in
ECPDATAOUT	equ	p5out
ECPDATADIR	equ	p5dir
ECPDATASEL	equ	p5sel
ECPCTRLIN	equ	p1in
ECPCTRLINDIR	equ	p1dir
ECPCTRLINSEL	equ	p1sel
ECPCTRLOUT	equ	p4out
ECPCTRLOUTDIR	equ	p4dir
ECPCTRLOUTSEL	equ	p4sel
HostCLK		equ	0x01
PeriphCLK	equ	0x01
PeriphAck	equ	0x02
AckReverse	equ	0x04
HostAck		equ	0x04
PeriphRequest	equ	0x10
ReverseRequest	equ	0x08
Active1284	equ	0x10
XFlag		equ	0x08

;;; LED defs
LED1	equ	0x10
LED2	equ	0x40

LED1ON	MACRO
	bis.b	#LED1,	&P2OUT
	ENDM
		
LED1OFF	MACRO
	bic.b	#LED1,	&P2OUT
	ENDM

LED2ON	MACRO
	bis.b	#LED2,	&P2OUT
	ENDM
		
LED2OFF	MACRO
	bic.b	#LED2,	&P2OUT
	ENDM

	if RUNSTATUS
LED1BLINK MACRO
; 	inc	&led1count
	jn	aus
	LED1ON
	jmp	weiter
aus:	LED1OFF
weiter:
	ENDM

LED2BLINK MACRO
	add	#0x100, &led2count
	jn	aus
	LED2ON
	jmp	weiter
aus:	LED2OFF
weiter:
	ENDM
	endif
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Startup
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	org 0xF800
	
startup:
	SECTION startup
	
	;; ---- stop watchdog and set stackpointer
	mov.w	#0x5A80,&WDTCTL
	mov.w	#stackstart,	SP

	;; ---- switch interrupts off
	dint
	mov.w	#0,	&IE1

	;; ---- setup timer module
	call	#osc_setup

	;; ---- set up LED port
	mov.b	#0,	&p2sel
	mov.b	#LED1|LED2, &p2dir
	mov.b	#0,	&p2out
	
	;; ---- set up the ports for the ECP
	mov.b	#0x0,	&ECPDATASEL ; ECP data lines

	mov.b	#0,	&ECPCTRLINSEL ; handshake lines from the PC
	mov.b	#2,	&ECPCTRLINDIR ; to the MSP (supports interrupts).

	mov.b	#0,	&ECPCTRLOUTSEL ; handshake lines from the MSP to the PC.
	mov.b	#0x1f,	&ECPCTRLOUTDIR

	mov.b	#0,	&ECPDATAOUT
	mov.b	#0,	&ECPDATAIN
	
	;; ---- initialize serial ports
	mov.b	#0x31,	&U0CTL	; 8N2-UART protocol 

	mov.b	#0x00,	&U0TCTL ; external clock

	bis.b	#U0TXE,	&ME1	; Enable TX0 module

	mov.b	#4,	&U0BR0	; Lo-Baudrate
	mov.b	#0,	&U0BR1	; Hi-Baudrate
	mov.b	#0,	&U0MCTL

	bis.b   #0x18,	&P3SEL	; UART1-TXD=P3.4
	bis.b   #0xf0,	&P3DIR

	bic.b	#1,	&U0CTL	; enable uart

	;; ---- timer a
	mov	#TASSEL1|TAMC1, &TACTL

	;; ---- set up dmx variables

	;; clear buffers
	mov	#stackstart-stacklen-out0buf, reg0
clrlp1:	mov.w	#0,	out0buf-2(reg0)
	decd	reg0
	jnz	clrlp1
	if 1
	mov	#512,	reg0
clrlp3:	mov.b	#0x20,	in0buf-2(reg0)
	dec	reg0
	jnz	clrlp3
	endif
	
	;; setup dmx config values
	mov	#0,	&dmxconfig
	mov.b	#0,	&out0sc
	mov.b	#0,	&out1sc
	mov	#512,	&out0max
	mov	#512,	&out1max
	mov	#88,	&out0brlen
	mov	#88,	&out1brlen
	mov	#8,	&out0mklen
	mov	#8,	&out1mklen
	
	eint

	call	#dmxout0_start

	if ECPDEBUG
	mov	#out0buf+20, ecpdebreg
	endif
	
	ENDSECTION		; section startup
;; --------------------------------------
	SECTION mainloop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; ECP Routines and Mainloop
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

checkforcommand MACRO
	bit	#HostAck<<8, returnreg
	jz	command_arrived
	bic	#0xff00, returnreg
	ENDM

breakforcommand MACRO
	bit	#HostAck<<8, returnreg
	jnz	dochnicht
	pop	&0xc00
	jmp	command_arrived
dochnicht:
	bic	#0xff00, returnreg
	ENDM

	;; wait for some action on ecp port
startup_ecp:
	call	#ecp_get8
	checkforcommand
	jmp	startup_ecp	

command_arrived:
	
	;; COMMAND 0:	slots
command0:
	cmp.b	#0x80,	returnreg
	jne	command1

	; get channel offset
	call	#ecp_get16
 	mov	returnreg, ecpouti

	; get channel values
channel_loop:
	call	#ecp_get8
	checkforcommand
	mov.b	returnreg, out0buf(ecpouti)
	inc	ecpouti
	jmp	channel_loop	

	;; COMMAND 1:	universe 0 size
command1:
	cmp.b	#0x81,	returnreg
	jne	command2

	call	#ecp_get16
	cmp	#24,	returnreg
	jge	ok0
	mov	#24,	returnreg
ok0:	cmp	#513,	returnreg
	jlo	ok1
	mov	#512,	returnreg
ok1:	mov	returnreg, &out0max

	jmp	channel_loop

	;; COMMAND 2:	universe 1 size
command2:
	cmp.b	#0x82,	returnreg
	jne	command3

	call	#ecp_get16
	cmp	#24,	returnreg
	jge	ok2
	mov	#24,	returnreg
ok2:	cmp	#513,	returnreg
	jlo	ok3
	mov	#512,	returnreg
ok3:	mov	returnreg, &out1max

	jmp	channel_loop

	;; COMMAND 3:	startcodes
command3:
	cmp.b	#0x83,	returnreg
	jne	command4

	call	#ecp_get16
	mov	returnreg, &out0sc

	jmp	channel_loop

	;; COMMAND 4:	break lengths
command4:
	cmp.b	#0x84,	returnreg
	jne	command5

	call	#ecp_get16
	mov.b	returnreg, &out0brlen
	swpb	returnreg
	mov.b	returnreg, &out1brlen

	jmp	channel_loop

	;; COMMAND 5:	mark lengths
command5:
	cmp.b	#0x85,	returnreg
	jne	command6

	call	#ecp_get16
	mov.b	returnreg, &out0mklen
	swpb	returnreg
	mov.b	returnreg, &out1mklen

	jmp	channel_loop

	;; COMMAND 6:	input universe config
command6:
	cmp.b	#0x86,	returnreg
	jne	command7

	call	#ecp_get16
	bit	#DMXOUT0ACTIVE, returnreg
	jz	weiter1
	call	#dmxout0_start
	
weiter1:
	mov	returnreg, &dmxconfig
	
	jmp	channel_loop

	;; COMMAND 7:	read request
command7:
	cmp.b	#0x87,	returnreg
	jne	command8

	call	#ecp_get16

	;; subcommand 1: get version string
	cmp	#1,	returnreg
	jne	subcom2

 	mov	#firmware_stringend-firmware_string, returnreg
  	call	#ecp_put16
	
	push	reg0	
	mov	#firmware_string, reg0 ; start of version string
loop1:	mov.b	@reg0+,	returnreg ; get next character
	call	#ecp_put8	; transmit character
	cmp.b	#0,	returnreg ; last character ?
	jne	loop1		; next character
	pop	reg0
	jmp	command7finished
		
subcom2:

command7finished:

command8:	
	jmp	startup_ecp

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; ecp_get8: retrieve one byte from ecp port
;; return: one byte in "returnreg"

ecp_get8:
	SECTION ecp_get8
	push	reg0

	;; prepare lines
	bic.b	#PeriphAck, &ECPCTRLOUT
	
	if ECPDEBUG
	mov	#out0buf+10, ecpdebreg
	mov.b	#0xdd,	@ecpdebreg
	inc	ecpdebreg
	endif

	;  wait for HostClk to go low
	mov	#0xffff, reg0
clklo:
	dec	reg0
	jz	abort
	if RUNSTATUS
	LED1BLINK
	endif
	bit.b	#HostCLK, &ECPCTRLIN
	jnz	clklo	

	if ECPDEBUG
	mov.b	#0x10,	@ecpdebreg
	inc	ecpdebreg
	endif

	;  set PeriphAck high
	bis.b	#PeriphAck, &ECPCTRLOUT

	; wait for rising edge of HostClk
	mov	#0xffff, reg0
clkhi:
	dec	reg0
	jz	abort
	if RUNSTATUS
	LED1BLINK
	endif
	bit.b	#HostCLK, &ECPCTRLIN
	jz	clkhi	

	if ECPDEBUG
	mov.b	#0x20,	@ecpdebreg
	inc	ecpdebreg
	endif

	; read data
	mov.b	#0x0,	&ECPDATADIR ; input
	mov.b	&ECPCTRLIN, returnreg ; Command or Data Cycle -> MSB
	swpb	returnreg

	mov.b	&ECPDATAIN, reg0	; Data -> LSB
	add	reg0, returnreg
	
	; set PeriphAck low
	bic.b	#PeriphAck, &ECPCTRLOUT

	if ECPDEBUG
	mov.b	#0x30,	@ecpdebreg
	inc	ecpdebreg
	endif

abort:	

	pop	reg0
	ret
	ENDSECTION

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; ecp_get16: get one wyde from ecp port
;; return: one wyde in "returnreg"
;; reg0 is trashed

ecp_get16:
	call	#ecp_get8		; MSB
	breakforcommand
	mov	returnreg, reg0
	swpb	reg0

	call	#ecp_get8		; LSB
	breakforcommand
	add	returnreg, reg0

	mov	reg0,	returnreg
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; ecp_put8: put one byte into ecp port
;; parameter: one byte in "returnreg"
;; return: the transmitted byte in "returnreg", which is unmodified from the above parameter

ecp_put8:
	SECTION ecp_put8
	push	reg0
	
	;; prepare input lines
	bis.b	#AckReverse|PeriphCLK, &ECPCTRLOUT
	bic.b	#PeriphAck, &ECPCTRLOUT
		
	if ECPDEBUG
	mov.b	#0xff,	@ecpdebreg
	inc	ecpdebreg
	endif

	;; Host sets nReverseChannel Low to request a reverse channel
	mov	#0xffff, reg0
revreqlo:
	dec	reg0
	jz	abort
	bit.b	#ReverseRequest, &ECPCTRLIN
	jnz	revreqlo

	if ECPDEBUG
	mov.b	#2,	@ecpdebreg
	inc	ecpdebreg
	endif

	;; Peripheral acknowledges reverse channel request via asserting nAckReverse low
	bic.b	#AckReverse, &ECPCTRLOUT

	;; Data is placed on lines by Peripheral
	mov.b	#0xff,	&ECPDATADIR ; output
	mov.b	returnreg, &ECPDATAOUT

	;; Data cycle is selected by Peripheral via PeriphAck high
	bis.b	#PeriphAck, &ECPCTRLOUT

	;; Valid data is indicated by the Peripheral setting PeriphClk low
	bic.b	#PeriphCLK, &ECPCTRLOUT

	;; Hosts sends its acknowledgment of valid data via HostAck high
	mov	#0xffff, reg0
hostackhi:
	dec	reg0
	jz	abort
	bit.b	#HostAck, &ECPCTRLIN
	jz	hostackhi
	
	if ECPDEBUG
	mov.b	#3,	@ecpdebreg
	inc	ecpdebreg
	endif
	
	;; Peripheral sets PeriphCLK hi to shift data into host
	bis.b	#PeriphCLK, &ECPCTRLOUT

	;; Host sends acknowledgment of data by HostAck low
	mov	#0xffff, reg0
hostacklo:
	dec	reg0
	jz	abort
	bit.b	#HostAck, &ECPCTRLIN
	jnz	hostacklo

	if ECPDEBUG
	mov.b	#4,	@ecpdebreg
	inc	ecpdebreg
	endif

abort:	
	if ECPDEBUG
	mov.b	#5,	@ecpdebreg
	inc	ecpdebreg
	endif

	;; set status lines to default
	mov.b	#0,	&ECPDATADIR ; input
	mov.b	#0,	&ECPDATAOUT
	bis.b	#AckReverse|PeriphAck|PeriphCLK, &ECPCTRLOUT
			
	pop	reg0	
	ret
	ENDSECTION

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; ecp_put16: put one wyde into ecp port
;; parameter: one wyde in "returnreg"
;; return: the transmitted wyde in "returnreg", which is unmodified from the above parameter
	
ecp_put16:
	SECTION ecp_put16
	swpb	returnreg
	call	#ecp_put8
	swpb	returnreg
	call	#ecp_put8
	ret
	ENDSECTION

	ENDSECTION			; mainloop
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; DMX Out 0
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

dmxout0_start:
	SECTION dmxout0_start
	;; if dmxout0 already running do nothing
	bit	#DMXOUT0ACTIVE, &dmxconfig
	jnz	ende
	;; enable dmxout0 by sending a byte and triggering the interrupt routine
	bis	#DMXOUT0ACTIVE, &dmxconfig
 	bis.b	#U0TXIE, &IE1	; enable uart0 tx interrupt
 	mov	&out0max, out0i
 	mov.b	&out0sc, &U0TXBUF

ende:	ret	
	ENDSECTION
	
dmxout0_handler:
	SECTION dmxout0
	
  	cmp	&out0max, out0i ; check if last byte was transmitted
   	jge	startbreak

	push	reg0
	mov.b	out0buf(out0i), reg0
	bit	#DMXIN0ACTIVE, &dmxconfig
	jz	out

	cmp.b	in0buf(out0i), reg0
	jhs	out
	mov.b	in0buf(out0i), reg0

out:	mov.b	reg0, &U0TXBUF ; send next byte
	inc	out0i
	pop	reg0
	reti

startbreak:

	;; check if dmxout0 still enabled
	bit	#DMXOUT0ACTIVE, &dmxconfig
	jz	ende
	
	bic.b	#U0TXIE, &IE1	; disable tx interrupts

	;; set up port for manual output
	bic.b	#0x10,	&P3OUT
	bis.b	#0x10,	&P3DIR
	bic.b	#0x10,	&P3SEL

	;; start timer a for break
	mov	&TAR,	&CCR0
	add	&out0brlen, &CCR0
	mov	#TACCIE, &CCTL0
	
	if RUNSTATUS
	LED2BLINK
	endif
	
ende:	reti
	ENDSECTION

dmxout0timer_handler:
	SECTION dmxout0timer_handler

	bis.b	#0x10,	&P3OUT	; mark after break
	bic	#TACCIE, &CCTL0	; stop timer a

	;; setup manual wait loop
	push	reg0
	mov	&TAR,	reg0
	add	&out0mklen, reg0
wait:	cmp	reg0, &TAR
	jl	wait
	pop	reg0

	bis.b	#0x10,	&P3SEL	; select port for uart
 	mov	#0,	out0i ; setup new dmx frame variables
	mov.b	&out0sc, &U0TXBUF ; send startcode
	bis.b	#U0TXIE, &IE1	; enable tx interrupts
	
	reti
	ENDSECTION
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Oscilator setup and failure
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

osc_setup:
	SECTION osc_setup

	bis.b	#XTS,	&BCSCTL1 ; select XT1
	bic.b	#XT2OFF,&BCSCTL1 ; enable XT2
	bic	#OscOff,SR	; enable XT1

	;; wait until osc stabilizes
	push	reg0
L1:	bic.b	#OFIFG,	&IFG1	;  Clear OscFault flag
	mov	#0x1000, reg0
wait:	dec	reg0
	jnz	wait
	bit.b	#OFIFG,	&IFG1	; check if oscs are working
	jnz	L1
	pop	reg0
	
	bic.b	#CPUOff|OscOff|SCG0|SCG1, SR ; enable XT1
	mov.b	#SELM1|SELM0, &BCSCTL2 ; select XT1 for main clock

 	bis.b	#OFIE,	&IE1	; enable osc_fail interrupt

	ret
	ENDSECTION
	
oscfail_handler:
	SECTION oscfail_handler

	dint

	;; we only need to test if we expect an NMI or Flash Violation interrupt
; 	bit.b	#OFIFG,	&IFG1
; 	jz	raus

	call	#osc_setup
	;; restart serial transmission
	bic	#DMXOUT0ACTIVE|DMXOUT1ACTIVE, &dmxconfig
	call	#dmxout0_start
	
raus:	eint
	reti
	
	ENDSECTION
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; strings
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

firmware_string: .byte "DMX43 V1.0 http://llg.cubic.org", 0
firmware_stringend:	
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; interrupt table
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

null_handler:	reti

	org 0xFFE0
	if 0			; MSP430x11xx
vec_01:		.word	null_handler
vec_02:		.word	null_handler
vec_Port1:	.word	null_handler
vec_Port2:	.word	null_handler
vec_05:		.word	null_handler
vec_06:		.word	null_handler
vec_07:		.word	null_handler
vec_08:		.word	null_handler
vec_TimerA0:	.word	null_handler
vec_TimerA1:	.word	null_handler
vec_Watchdog:	.word	null_handler
vec_compA:	.word	null_handler
vec_13:		.word	null_handler
vec_14:		.word	null_handler
vec_NMI:	.word	null_handler
vec_RES:	.word	null_handler
	endif

	if 0			; MSP430x12x
vec_01:		.word	null_handler
vec_02:		.word	null_handler
vec_P1:		.word	null_handler
vec_P2:		.word	null_handler
vec_05:		.word	null_handler
vec_06:		.word	null_handler
vec_UART0_TX:	.word	null_handler
vec_UART1_RX:	.word	null_handler
vec_TimerA0:	.word	null_handler
vec_TimerA1:	.word	null_handler
vec_Watchdog:	.word	null_handler
vec_compA:	.word	null_handler
vec_13:		.word	null_handler
vec_14:		.word	null_handler
vec_NMI:	.word	null_handler
vec_RES:	.word	null_handler
	endif

	if 1			; MSP430x13x || MSP430x14x
vec_01:		.word	null_handler
vec_Port2:	.word	null_handler
vec_UART1_TX:	.word	null_handler
vec_UART1_RX:	.word	null_handler
vec_Port1:	.word	null_handler
vec_TimerA1:	.word	null_handler
vec_TimerA0:	.word	dmxout0timer_handler
vec_ADC:	.word	null_handler
vec_UART0_TX:	.word	dmxout0_handler
vec_UART0_RX:	.word	null_handler
vec_Watchdog:	.word	null_handler
vec_compA:	.word	null_handler
vec_TimerB1:	.word	null_handler
vec_TimerB0:	.word	null_handler
vec_NMI:	.word	oscfail_handler
vec_RES:	.word	startup
	endif
